#define CHANLEN			5	/* default channel length (5) */

/* Possible status a commitment can be in */
mtype = { m_idle, m_active, m_inactive };

/* Possible states a commitment can be in */
mtype = {  cc_idle, cc_active, cc_base, cc_cancelled, cc_released, cc_pending, cc_discharged};
/* message sent to cc_process */
mtype = {  cc_create, cc_cancel, cc_release, cc_assign, cc_delegate, cc_discharge, cc_query,  cc_update};

/* There are a commitment has:
debtor: a channel able to reach.
creditor: a channel to the creditor.
condition p: pre-condition
condition q: post-condition
state: can be created, cancelled, assigned, delegated, done
isactive: m_idle, m_active, m_inactive 
*/
typedef   Commitment {
	/* Channel to Reach Debtor */
	chan debtor;
	
	/*Channel to Reach Creditor */
	chan creditor;
	
	/* store the status of the commitment,
	   check whether it is dependent on other commitment,
	   by default it is 0. If it is dependent on others, 
	   the state will become pending. 
	*/	
	mtype state;
	

	/* the dependent commitment, by default it is 0 meaning 
	   no dependency. */
	byte  pcid;
}

/*Commitment array:
as Promela can not dynamically create new objects. We have to
create them from the beginning. */
Commitment cc[CHANLEN] ;

show short c_cnt=1; /* Commitment Counter starts from index 1*/

/* message sent from cc_process */
mtype = { cc_cid };

/* For commitment creation */
chan to_cc_create =  [CHANLEN] of 	{mtype, chan, chan, chan};

/* For update commitment information mtype, chan, cid, state*/
chan to_cc_update = [CHANLEN] of   { mtype, chan, byte, mtype};

/* For commitment management */
chan to_cc_mgt = [CHANLEN] of   { mtype, chan, byte, chan};

/*
cancel(x,c): action of c's debtor x, x is represented by the channel.
So the cancel messages should have a slot for channel along with the commitment
id; so its message format should be (byte, chan)

release(y,c) action of c's creditor y
So is for release message.

assign(y,z, c) replaces y with z as c's creditor.
delegate(x, z, c) replaces x with z as c's debtor
The format of above two should be (byte, chan, chan).

In fact either x or y can be infered by the commitment.
So we do not need to send it. However, if we do send it, we can use it check whether
it is the right one sending the message.
the format could be (byte, chan) or (byte, chan, chan)

discharge(x, c): c s terminated by x fulfilling it. It looks like discharge is maintained
by the commitment system.
*/

active proctype CCProcess( ) 
{
      chan debtor, creditor;
      chan who;
      chan tempchan;
      byte tmp1, tmp2;
      byte tmp3, tmp4;
      mtype msg, msg2;
      byte i;
	
      /* Initialize the Commitment structure */
      
     do
     	:: i< CHANLEN ->
     	   d_step {	
		cc[i].debtor = 0;
		cc[i].creditor = 0;
		cc[i].state = cc_idle;
		cc[i].pcid =0;
		i = i+1;
	   }
	:: i >= CHANLEN ->
		break;
     od;	
     


endccp: do
         /* check whether there are create CC messages */
	 :: to_cc_create?[cc_create] ->
		to_cc_create?msg, who, debtor, creditor;
		atomic {
		cc[c_cnt].debtor = debtor;
		cc[c_cnt].creditor = creditor;
		cc[c_cnt].state = cc_active;
		cc[c_cnt].creditor!cc_cid(c_cnt);
		who! cc_cid(c_cnt);
		
		c_cnt = c_cnt +1;
		}
		
	
	:: to_cc_mgt?[msg]->
		to_cc_mgt?msg, who, tmp1, tempchan;
		if
		:: msg == cc_cancel ->
			/* cancel the commitment */			
			cc[tmp1].state = cc_cancelled;
			cc[tmp1].creditor!cc_cancel, tmp1;
		:: msg == cc_base ->
			if
			 :: cc[tmp1].state == cc_active ->
				cc[tmp1].state = cc_base;
			 :: else ->
			    skip;
			fi
		:: msg == cc_release ->
			/* release the commitment, we can check whether it is from
			the right process using the channel*/
			cc[tmp1].state = cc_released;
			cc[tmp1].debtor!cc_release(tmp1);


		:: msg == cc_assign ->
			/* assign the commitment */
			cc[tmp1].creditor = tempchan;
			cc[tmp1].state = cc_pending;
		:: msg == cc_delegate ->
			/* delegate the commitment */
			cc[tmp1].debtor = tempchan;
			cc[tmp1].state = cc_pending;
			cc[tmp1].creditor!cc_delegate(tmp1);


		:: msg == cc_discharge ->			
			/* change the state of CC to terminate*/
			cc[tmp1].state = cc_discharged;			
		fi;     	
     od;

}

