/* 
Include the Common Commitment Processor
It will manage the commitments.
*/

#include "ccp.pml"

/* Protocol Specific Part: shared by all role processes*/

/* Channels used to maintain the states of cmmitments */
chan to_buyer_cc = [1] of {mtype, byte};
chan to_seller_cc = [1] of {mtype, byte};
chan to_seller2_cc = [1] of {mtype, byte};

/*messages shared by all */
mtype = { reqForQuote, quote, acceptQuote };


/* Role Process: Seller Model */
chan  to_seller= [5] of { mtype, chan, byte, byte};
chan  to_seller2= [0] of { mtype, chan, byte, byte};

/*Shipping and Payment */

mtype = { paymentInfo,  authReq, authOK, captureReq, captured, receipt };

chan  to_payee = [CHANLEN] of { mtype, chan, byte, byte};
chan  to_gateway  = [CHANLEN] of { mtype, chan, byte, byte, byte };

mtype = { shipInfo,   reqForShipOptions,  shipperOptionQuote, senderOptionQuote, 
chooseOption, shipOrder, shipment, confirm };

chan  to_sender = [5] of { mtype, chan, byte, byte};
chan  to_shipper  = [CHANLEN] of { mtype, chan, chan, byte, byte, byte };
bit b_paid = 0;
bit s_paid = 0;
bit b_received =0;
bit s_shipped=0; 

active proctype Shipper()
{
    chan from;
    chan to;
     byte shipAddress;
     byte item;
     byte shipOption;
     byte shipQuote;
idle:
      to_shipper?reqForShipOptions(from, to, shipAddress, item);
      shipOption = 1;
      shipQuote  = item/2;
      from!shipperOptionQuote(shipOption, shipQuote);
      to_shipper?shipOrder ( from, to, item, shipOption, shipAddress);
      to!shipment(item);
      to?confirm(item);    
      from!confirm(item);
   goto idle
 }

active proctype Gateway()
{
    chan who;
    byte cardNO;
    byte expDate;
    byte amount;
    byte tokenNO;
    byte token;   

end:
      to_gateway?authReq(who, cardNO, expDate, amount);
      tokenNO = _pid ;

      who!authOK( tokenNO, amount );
      to_gateway?captureReq(who, token);
     who!captured(amount);
   goto end
 }

active proctype Seller()
{
    byte itemID;
    byte itemPrice;

   /* payment variables*/
    byte cardNO;
    byte expDate;
    byte item;
    byte amount ;
    byte tokenNO;
    byte authorizedAmount;
    byte capturedAmount;
    byte totalCaptured = 0;

     /*ship variables*/
    byte shipAddress;
     byte shipOption;
     byte shipperQuote;
     byte senderQuote;
     byte confirmedItem;
    
    chan who;

    mtype msg;
    byte var1, var2;
    bool done = false;


  byte cid, cid1, cid2; 
  byte scid; /*Seller commitment id*/
  chan me = [0] of { mtype, byte};

accept:  
    do
      /* check whether there is business messages */
      :: to_seller?[msg] -> 
        to_seller?msg, who, var1, var2;
	if
	   ::msg == reqForQuote ->
		itemID = var1;
		itemPrice =  2 * itemID;
	      	who!quote(itemID, itemPrice);
 	  :: msg == acceptQuote ->
		
  	 	to_cc_create!cc_create(me, to_seller_cc, to_buyer_cc);
		me?cc_cid(scid);
  
  		
	  
		/*to_cc_mgt!cc_delegate(me, scid, to_seller2_cc); */
	Pay:
	       amount = itemPrice;
	       item = itemID;

	       /**Payment**/
	       to_payee?paymentInfo(who, cardNO, expDate);
	       to_gateway!authReq (me, cardNO, expDate, amount) ;
	       me?authOK(tokenNO, authorizedAmount);

	       who!receipt(authorizedAmount);
	       to_gateway!captureReq(me, tokenNO);
	       me?captured(capturedAmount);       
	       totalCaptured = totalCaptured + capturedAmount;

	       /* received the payment, update its commitment */
	       /* Update the pre part of condition */
		to_cc_mgt!cc_base(me, scid);

		/* have to wait until that buyer commitment id is received	   		
		*/

		if
		  :: (cid1==0) ->
			   to_seller_cc?cc_cid(cid1);
		  :: else -> skip;
		fi;

		to_cc_mgt!cc_discharge(me, cid1);



	       printf("Current total captured amount for payee: %d\n", totalCaptured);

		/**shipping**/
	       to_sender?shipInfo (who, shipAddress);
	       to_shipper!reqForShipOptions( me, who, shipAddress, item);
	       me?shipperOptionQuote(shipOption, shipperQuote);
	       /* seller can add a little premium here*/
	       senderQuote = shipperQuote + 1;
	       who!senderOptionQuote(shipOption, senderQuote);
	       to_sender?chooseOption( who, shipOption, senderQuote);
	       to_shipper!shipOrder( me, who, item, shipOption, shipAddress);

	       to_cc_mgt!cc_discharge(me, scid);     
   		
   	       /*work is done */
   	       break;

	fi;
	
      :: to_seller_cc?[msg] ->
   	to_seller_cc? msg, cid;
   	if
	 :: msg == cc_cid ->
   	    cid1 = cid;
   	 :: msg == cc_cancel ->
   	    skip;
   	 :: msg == cc_release ->
   	    skip;      
   	fi  
   	
od;

end: 
skip;

 }

/* Role Process: Buyer Model */

chan to_buyer = [0] of { mtype, chan, byte, byte};
 active proctype Buyer()
{



     byte itemID;
     byte itemPrice;
     byte cardNO =123;
     byte expDate = 234;
     byte expectedAmount ;
     byte expectedItem;
     byte paidAmount;
      byte shipOption;
     byte senderQuote;
     byte item;
   byte shipAddress= 111;
  chan me = [0] of { mtype, byte, byte};
  bool done;
  done = false;
  byte cid1; /*Used the store the first commitment from others */ 
  byte cid, cid2;
  byte bcid;/*Used to store the buyer commitment ID*/
  mtype msg;
  do
  
   :: !done->
	itemID = 100;
	to_seller!reqForQuote(me, itemID) ->
	me?quote(itemID, itemPrice);
	

	
	/*to_seller!cc_cid(me, bcid);*/
	to_seller!acceptQuote ( me, itemID,  itemPrice);

	/* Create a Commitment that say if seller ships, buyer will pay */
	to_cc_create!cc_create(me, to_buyer_cc, to_seller_cc);
	/* Receive the cid for this commitment */
	me?cc_cid(bcid);
	
	/*Cancel the commitment to buy */
	/*to_cc_mgt!cc_cancel(me, bcid);*/
	
	/*payment*/
	to_payee!paymentInfo (me, cardNO, expDate);
	me?receipt(paidAmount);
	
	/*shipping */
	to_sender!shipInfo(me, shipAddress);
	me?senderOptionQuote(shipOption, senderQuote);
	to_sender!chooseOption( me, shipOption, senderQuote);
	me?shipment(item);
	to_cc_mgt!cc_discharge(me, bcid);
	
	done = true;
	break
        /*query whether there is message from ccp */
   :: to_buyer_cc?[msg] ->
   	to_buyer_cc? msg, cid;
   	if
	 :: msg == cc_cid ->
   	    cid1 = cid;
   	 :: msg == cc_cancel ->
   	    skip;
   	 :: msg == cc_delegate ->
   	    /*The implied commitment is delegated to some one else */
   	    /* Release this commitment */
   	    to_cc_mgt!cc_release(me, cid1);
   	    skip;      
   	 :: msg == cc_release ->
   	    skip;   
   	fi
  od;
 }
